package net.codinux.banking.epcqrcode


open class EpcQrCodeConfig(

    /**
     * Name of the Receiver. Mandatory.
     * Max. 70 characters
     */
    open val receiverName: String,

    /**
     * Receiver's IBAN. Mandatory.
     * Max. 34 bytes.
     */
    open val iban: String,

    open val bic: String? = null,

    amount: String? = null,

    open val reference: String? = null,

    /**
     * Beneficiary to originator information.
     *
     * The text contained in Display must be shown to the user after decoding and serves a
     * short textual description of what the user is going to initiate.
     * This text must not be forwarded with the data handed over to the payment system.
     */
    open val informationForUser: String? = null,

    open val purposeCode: String? = null,

    open val qrCodeHeightAndWidth: Int = EpcQrCode.DefaultHeightAndWidth,

    open val encoding: EpcQrCodeCharacterSet = EpcQrCodeCharacterSet.UTF_8,

    open val imageFormat: ImageFormat = ImageFormat.PNG

) {

    companion object {

        fun ensureAmountFormat(amount: String?): String? {
            return amount?.replace(',', '.')
        }

    }


    open val amount: String? = ensureAmountFormat(amount)


    // convenience constructors for languages not supporting default values (like Swift and JavaScript)

    constructor(receiverName: String, iban: String, amount: String?, reference: String?) : this(receiverName, iban, null, amount, reference)

    constructor(receiverName: String, iban: String, bic: String?, amount: String?, reference: String?) : this(receiverName, iban, bic, amount, reference, null)

}