package net.codinux.banking.epcqrcode

open class EpcQrCodeStringFormatter {

  companion object {
    const val BitIsSetColor = "█"

    const val BitIsNotSetColor = " "

    const val SmallSizeUpperBitSetColor = "▀"

    const val SmallSizeLowerBitSetColor = "▄"

  }


  open fun asString(epcQrCode: EpcQrCode, border: Int = 0, invertColors: Boolean = false): String {
    val bitIsSetColor = if (invertColors) BitIsNotSetColor + BitIsNotSetColor else BitIsSetColor + BitIsSetColor
    val bitIsNotSetColor = if (invertColors) BitIsSetColor + BitIsSetColor else BitIsNotSetColor + BitIsNotSetColor
    val height = epcQrCode.height
    val width = epcQrCode.width
    val string = StringBuilder(height * width)

    printBorderAboveOrBelow(border, width, string, bitIsNotSetColor)

    for (y in 1 until height) {
      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      for (x in 1 until width) {
        if (isBitSet(epcQrCode, x, y)) {
          string.append(bitIsSetColor)
        } else {
          string.append(bitIsNotSetColor)
        }
      }

      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      string.appendLine()
    }

    printBorderAboveOrBelow(border, width, string, bitIsNotSetColor)

    return string.toString()
  }

  open fun asSmallString(epcQrCode: EpcQrCode, border: Int = 0, invertColors: Boolean = false): String {
    val bitIsSetColor = if (invertColors) BitIsNotSetColor else BitIsSetColor
    val bitIsNotSetColor = if (invertColors) BitIsSetColor else BitIsNotSetColor
    val upperBitSetColor = if (invertColors) SmallSizeLowerBitSetColor else SmallSizeUpperBitSetColor
    val lowerBitSetColor = if (invertColors) SmallSizeUpperBitSetColor else SmallSizeLowerBitSetColor
    val height = epcQrCode.height
    val width = epcQrCode.width
    val string = StringBuilder((height + border) * (width + border))

    printBorderAboveOrBelow(border, width, string, bitIsNotSetColor)

    for (y in 1 until height - 1 step 2) {
      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      for (x in 1 until width) {
        val currentRowBit = epcQrCode.getColorCodeAt(x, y)
        val nextRowBit = epcQrCode.getColorCodeAt(x, y + 1)

        if (currentRowBit == nextRowBit) {
          if (isBitSet(currentRowBit)) {
            string.append(bitIsSetColor)
          } else {
            string.append(bitIsNotSetColor)
          }
        } else {
          if (isBitSet(currentRowBit)) {
            string.append(upperBitSetColor)
          } else {
            string.append(lowerBitSetColor)
          }
        }
      }

      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      string.appendLine()
    }

    if (height % 2 == 0) { // if last row is odd
      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      val y = height - 1
      for (x in 1 until width) {
        if (isBitSet(epcQrCode, x, y)) {
          string.append(upperBitSetColor)
        } else {
          string.append(bitIsNotSetColor)
        }
      }

      printBorderBeforeOrAfter(border, string, bitIsNotSetColor)

      string.appendLine()
    }

    printBorderAboveOrBelow(border, width, string, bitIsNotSetColor)

    return string.toString()
  }

  protected open fun isBitSet(epcQrCode: EpcQrCode, x: Int, y: Int): Boolean {
    val qrCodeBit = epcQrCode.getColorCodeAt(x, y)

    return isBitSet(qrCodeBit)
  }

  protected open fun isBitSet(qrCodeBitColorCode: Int): Boolean {
    // black = -1, white = -16777216
    return qrCodeBitColorCode < -1
  }


  private fun printBorderAboveOrBelow(border: Int, width: Int, string: StringBuilder, bitIsNotSetColor: String) {
    for (y in 0 until border) {
      for (x in 1 until width + 2 * (border * 2)) {
        string.append(bitIsNotSetColor)
      }

      string.appendLine()
    }
  }

  private fun printBorderBeforeOrAfter(border: Int, string: StringBuilder, bitIsNotSetColor: String) {
    for (x in 0 until (border * 2)) {
      string.append(bitIsNotSetColor)
    }
  }

}